;*******************************************************************************
;* Example program for the MiniRISC CPU.                                       *
;* This example shows how to use indirect jump to implement a jump table.      *
;*******************************************************************************
DEF     LD      0x80        ; LED register (read/write)
DEF     SW      0x81        ; DIP switch register (read-only)

start:
    mov     r1, #0          ; r1 stores the state of the LEDs.
    mov     r2, #0          ; r2 stores the current state of the switches.
    mov     r3, #0          ; r3 stores the previous state of the switches.
    mov     r4, #0xf0       ; r4 stores the ####____ pattern.
    mov     r5, #jmp_tbl    ; r5 stores the start address of the jump table.

check_sw:
    mov     r3, r2          ; Store the previous state of the switch.
    mov     r2, SW          ; Read the current state of the switch.
    and     r2, #0x03       ; Only SW0 and SW1 are used.
    sub     r0, r2, r3      ; Check whether the switch has been changed.
    jz      no_change
    mov     r1, #0          ; Set the state of the LEDs to 0.
no_change:
    mov     LD, r1          ; Update the LEDs.

    add     r15, r5, r2     ; Add the offset to the jump table start address.
    jsr     (r15)           ; Call the selected function using the jump table.
    jmp     check_sw

jmp_tbl:                    ; Jump table
    jmp     function1
    jmp     function2
    jmp     function3
    jmp     function4

function1:                  ; LEDs: ######## <-> ________
    xor     r1, #0xff
    rts

function2:                  ; LEDs: ####____ <-> ____####
    sub     r0, r1, r4      ; Check the current pattern.
    jz      func2_p2
func2_p1:
    mov     r1, #0xf0       ; Set the ####____ pattern.
    rts
func2_p2:
    mov     r1, #0x0f       ; Set the ____#### pattern.
    rts

function3:                  ; Johnson counter (left shift)
    sl0     r1, r1
    jc      func3_end 
    or      r1, #0x01
func3_end:
    rts

function4:                  ; Johnson counter (right shift)
    sr0     r1, r1
    jc      func4_end 
    or      r1, #0x80
func4_end:
    rts
