﻿//******************************************************************************
//* Simple assembler for the MiniRISC CPU.                                     *
//*                                                                            *
//* Written by: Tamas Raikovich                                                *
//* Version   : 2.0                                                            *
//******************************************************************************
using System;
using System.Collections.Generic;
using System.Text;

namespace MiniRISC_as
{
    class Program
    {
        static void MessageEventHandler(object sender, string msg)
        {
            Console.WriteLine(msg);
        }

        static void Main(string[] args)
        {
            //Write a message.
            Console.WriteLine("MiniRISC assembler v2.1");
            Console.WriteLine("Copyright (C) 2011 LOGSYS, Tamas Raikovich");
            Console.WriteLine();

            //Check the parameters.
            if (args.Length != 1)
            {
                Console.WriteLine("Usage: minirisc-as filename.s");
                Console.WriteLine();
                return;
            }

            using (Assembler assembler = new Assembler())
            {
                try
                {
                    //Set the message event handler.
                    assembler.Message += MessageEventHandler;

                    //Open the source file.
                    assembler.Open(args[0]);
                    //Process the source file.
                    assembler.Assemble();

                    //Write a message.
                    Console.WriteLine("MiniRISC assembler completed with " + assembler.NumOfErrors.ToString() + " error(s).");
                    Console.WriteLine();
                }
                catch (Exception ex)
                {
                    //Write a message.
                    Console.WriteLine("Error: " + ex.Message);
                    Console.WriteLine();
                }
            }
        }
    }
}
