//******************************************************************************
//* 256 x 16 bit program memory. Can be updated through the JTAG interface.    *
//******************************************************************************
module prg_memory(
   input  wire [7:0]  prg_mem_addr,    //Address input
   output wire [15:0] prg_mem_data,    //Data output
   output wire        jtag_rst         //Reset output (reset during JTAG update)
);

//******************************************************************************
//* JTAG interface.                                                            *
//******************************************************************************
wire        drck1;
wire        update;
wire        prg_mem_clk;
wire        shr_clk;
reg  [23:0] shr;
wire        shr_din;

//Boundary scan interface.
BSCAN_SPARTAN3 bscan_interface(
   .CAPTURE(),          //CAPTURE output from TAP controller
   .DRCK1(drck1),       //Data register output for USER1 functions
   .DRCK2(),            //Data register output for USER2 functions
   .RESET(),            //Reset output from TAP controller
   .SEL1(jtag_rst),     //USER1 active output
   .SEL2(),             //USER2 active output
   .SHIFT(),            //SHIFT output from TAP controller
   .TDI(shr_din),       //TDI output from TAP controller
   .UPDATE(update),     //UPDATE output from TAP controller
   .TDO1(1'b0),         //Data input for USER1 function
   .TDO2(1'b0)          //Data input for USER2 function
);

//Clock buffers.
BUFG shr_clk_bufg(.I(drck1), .O(shr_clk));
BUFG mem_clk_bufg(.I(update), .O(prg_mem_clk));

//Shift register.
always @(posedge shr_clk)
begin
   shr <= {shr_din, shr[23:1]};
end


//******************************************************************************
//* 256 x 16 bit distributed RAM.                                              *
//******************************************************************************
(* ram_style = "distributed" *)
reg  [15:0] prg_mem [255:0];
wire [7:0]  wr_address  = shr[23:16];
wire [15:0] prg_mem_din = shr[15:0];

always @(posedge prg_mem_clk)
begin
   prg_mem[wr_address] <= prg_mem_din;
end

assign prg_mem_data = prg_mem[prg_mem_addr];


endmodule
