//******************************************************************************
//* MiniRISC CPU                                                               *
//*                                                                            *
//* 16 x 8 bit register file with 1 write and 2 read ports. R0 is always 0.    *
//* Distributed RAM is used in this version.                                   *
//******************************************************************************
module reg_file_1w2r_lut(
   //Clock signal.
   input  wire       clk,
   
   //Write port.
   input  wire [3:0] wr_addr,       //Register address
   input  wire       wr_en,         //Write enable signal
   input  wire [7:0] wr_data,       //Input data
   
   //Read port A.
   input  wire [3:0] rd_addr_a,     //Register address
   output wire [7:0] rd_data_a,     //Register data output
   
   //Read port B.
   input  wire [3:0] rd_addr_b,     //Register address
   output wire [7:0] rd_data_b      //Register data output
);

//******************************************************************************
//* Declaring the 16 x 8 bit memory.                                           *
//******************************************************************************
(* ram_style = "distributed" *)
reg [7:0] reg_file [15:0];


//******************************************************************************
//* Implementing the write port. Write operations are synchronous.             *
//******************************************************************************
always @(posedge clk)
begin
   if (wr_en)
      reg_file[wr_addr] <= wr_data;
end


//******************************************************************************
//* Implementing the read port A. Read operations are asynchronous.            *
//* If the reg. address is 0 (R0 is selecres), the read data will be 0.        *
//******************************************************************************
assign rd_data_a = (rd_addr_a == 4'd0) ? 8'd0 : reg_file[rd_addr_a];


//******************************************************************************
//* Implementing the read port A. Read operations are asynchronous.            *
//* If the reg. address is 0 (R0 is selecres), the read data will be 0.        *
//******************************************************************************
assign rd_data_b = (rd_addr_b == 4'd0) ? 8'd0 : reg_file[rd_addr_b];


endmodule
